package pl.edu.pwr.example.geoClient;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.WebApplicationType;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.builder.SpringApplicationBuilder;

import java.awt.EventQueue;
import java.util.Collections;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;

import pl.edu.pwr.example.geoClient.data.Data;
import pl.edu.pwr.example.geoClient.data.Quote;
import pl.edu.pwr.example.geoClient.gui.MainView;

@SpringBootApplication
public class GeoClientApplication {
	private static final Logger log = LoggerFactory.getLogger(GeoClientApplication.class);

	public static void main(String[] args) {
		SpringApplicationBuilder builder = new SpringApplicationBuilder(GeoClientApplication.class);
		builder.headless(false).web(WebApplicationType.NONE);

		ConfigurableApplicationContext context = builder.run(args);

		RestTemplate restTemplate = context.getBean(RestTemplate.class);

		// request url
		String url = "https://wft-geo-db.p.rapidapi.com/v1/geo/cities";

		// create headers
		HttpHeaders headers = new HttpHeaders();
		headers.add("x-rapidapi-key", "bf9d36129cmshef5a5927c02428cp16af81jsnd43a783XXXX");
		headers.add("x-rapidapi-host", "wft-geo-db.p.rapidapi.com");
		headers.setContentType(MediaType.APPLICATION_JSON);
		headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));

		// build the request
		HttpEntity<String> request = new HttpEntity<String>(headers);

		// make an HTTP GET request with headers
		ResponseEntity<Quote> response = restTemplate.exchange(url, HttpMethod.GET, request, Quote.class);

		// check response
		if (response.getStatusCode() == HttpStatus.OK) {
			System.out.println("Request Successful.");
			System.out.println(response.getBody());
		} else {
			System.out.println("Request Failed");
			System.out.println(response.getStatusCode());
		}

		Quote quote = response.getBody();
		log.info(quote.toString());

		EventQueue.invokeLater(() -> {

			var frame = context.getBean(MainView.class);
			frame.setVisible(true);
			StringBuilder sb = new StringBuilder();
			for (Data d : quote.getData()) {
				sb.append(d.toString()+"\n");
			}
			frame.textArea.setText(sb.toString());
		});

	}

	@Bean
	public RestTemplate restTemplate(RestTemplateBuilder builder) {
		return builder.build();
	}

	@Bean
	public MainView mainView() {
		return new MainView();
	}

}
